# Builds the file scripts/basin_start.sh 
# The first argument to this script should be the python install directory as determined
# by configure
# The second argument should be the install path for VisIt, if this is argument is the empty string,
# it will be assumed that visit support is not built and the environment will not be set for VisIt

echo "#! /bin/bash

hostname=\`hostname\`

function print_usage 
{
    echo \"This script will start the BASIN remote environment. It should 
be run on your cluster. The controller will be launched on this machine
while the engines will be started using mpiexec. You should have your 
MPI environment created before using this command.

Arguments
    -n --nodes  number of nodes to bring up. This argument is required
    -c --client Client machine to send configuration information to. This will
                store a connection file in ~/.basin/\$(hostname)_remote.furl on 
                the machine passed as an argument to this flag.\"
}

num_nodes=0
client_hostname=\"False\"
visit_dir=$2

while [ \"\$1\" != \"\" ]; do
    case \$1 in
        -n | --nodes )
            shift
            num_nodes=\$1
            ;;
        -c | --client )
            shift
            client_hostname=\$1
            ;;
        * )
            print_usage
            exit
            ;;
    esac
    shift
done

if [ \$num_nodes -eq 0 ]; then
    print_usage
    exit
fi

# Check if we can source the environment variable settings
if [ -f basin_remote_env.sh ]; then
    source basin_remote_env.sh
    env_found=\"basin_remote_env.sh\"
else
    env_found=\"No environment file found.\"
fi

# Create file containing path to client scripts - James
if ! [ -d \$HOME/.basin ]; then
	mkdir \$HOME/.basin
fi
if ! [ -d \$HOME/.basin/host ]; then
        mkdir \$HOME/.basin/host
fi
script_path=\$0
if [ -L \$script_path ] ; then
	script_path=\`ls -l \$script_path | gawk '{ print \$11 }'\`
else
	script_path=\`pwd\`/\$script_path
fi
script_path=\`dirname \$script_path\`
echo \"== Basin Client Paths ==\"
echo \"Scripts Path: \"\$script_path
echo \"  VisIt Path: \"\`which visit\`
echo \"========================\"
echo \$script_path > \$HOME/.basin/host/client_script.path
echo \"\`which visit\`\" > \$HOME/.basin/host/visit.path
echo \"========================\"
echo \"Using environment settings from file: \$env_found\"
echo ""


# If visit_dir has been set, then set the environment vars accordingly
if [ -n \"\$visit_dir\" ]; then
    LD_LIBRARY_PATH=\"\$visit_dir/lib:\$LD_LIBRARY_PATH\"
    VISITPLUGINDIR=\"\$visit_dir/plugins\"
fi

# Set the environment for the BASIN kernel libraries
if [ \"\$BASIN_LIB_DIR\" ]; then
    LD_LIBRARY_PATH=\"\$BASIN_LIB_DIR:\$LD_LIBRARY_PATH\"
fi

# Make sure all the necessary environment variables are sent to the cluster
env_args=\"-env PYTHONPATH $1:\$PYTHONPATH\"

# LD_LIBRARY_PATH and VISITPLUGINDIR are needed for visit support. These may 
# not already be defined, so test for existence before using them
if [ \"\$LD_LIBRARY_PATH\" ]; then
    env_args=\$env_args\" -env LD_LIBRARY_PATH \$LD_LIBRARY_PATH\"
fi

if [ \"\$VISITPLUGINDIR\" ]; then
    env_args=\$env_args\" -env VISITPLUGINDIR \$VISITPLUGINDIR\"
fi

# Launch with default ports

ipcontroller &
controller_pid=\$!
sleep 3

# Test that the controller started ok
if [ \"x\"\`jobs | awk '{print \$2}'\` = 'xRunning' ]; then
    mpiexec -n \$num_nodes \$env_args ipengine &
    engine_pid=\$!
else
    echo \"Error launching the controller, exiting.\"
    exit
fi

# create a sig handler that will kill our two children
trap \"{ kill -9 \$controller_pid; kill -9 \$engine_pid; exit; }\" SIGINT SIGKILL

ipcon_mec_path=\"\$HOME/.ipython/ipcontroller-mec.furl\"
furl_file=\"\${hostname}_remote.furl\"
furl_file_path=\"~/.basin/\$furl_file\"

# Copy the connection file locally
echo \"Connection file: \$furl_file_path\"
cp \$ipcon_mec_path \$HOME/.basin/\$furl_file

# Copy the connection configurations to the client machine, if necessary
if [ \"\$client_hostname\" != \"False\" ]; then
    echo \"Copying connection file to: \$client_hostname:~/.basin/\${hostname}_basin.furl\"
    scp_string=\"\$ipcon_mec_path \$client_hostname:\$furl_file_path\"
    \`scp \$scp_string\`
fi

# make sure the script doesn't exit before its children do 
wait
    
" > basin_start.sh 
chmod +x basin_start.sh
