#ifndef __BASIN_CONTAINER_CONVERSION_H__
#define __BASIN_CONTAINER_CONVERSION_H__

#include <Python.h>
#include <boost/python.hpp>
#include <boost/variant.hpp>

#include "basin_helpers.h"

/*
 * Helper function to test that elements of an iterable type
 * can be extracted safely.
 * @params obj_itr  Expects a Python Object iterator.
 */
template<typename T>
struct test_elements {
    static bool test(PyObject *obj_itr) {
        PyObject *item;

        while(item = PyIter_Next(obj_itr)) {
            boost::python::extract<T> obj_proxy(item);
            if(!obj_proxy.check()) return false; //tests that extraction for T is allowed
        }

        return true;
    }
};

/*
 * The following two structs facilitate the conversion of C++ containers
 * to and from Python types. These functions will be called implicitly once
 * they have been registered as converters. They are designed to be used
 * for automatic conversion when containers are used as arguments to C++
 * functions or as return values from C++ functions.
 * As an example to support arguments of type vector<double> you would
 * need the following in your BOOST_PYTHON_MODULE block:
 *      container_from_list<vector<double> >();
 *
 * To support return types of vector<double> you would need the following
 * in your BOOST_PYTHON_MODULE block:
 *      to_python_converter<vector<double>, container_to_list<vector<double> > >();
 */

/* 
 * Converts to C++ types from a Python List or Tuple. 
 * This function will work with any one dimensional data structure
 * that contains types that Boost Python knows how to convert
 * to their equivalent C++ types
 */
template<typename T>
struct container_from_list
{
    typedef typename T::value_type T_v_t;
    // A new object must be constructed for every different container<T> type that is needed
    // for instance to use vectors of int and strings one would need to type code:
    //     container_from_list<vector<int> >();
    //     container_from_list<vector<string> >();
    container_from_list() {
        boost::python::converter::registry::push_back(
                &convertible,
                &construct,
                boost::python::type_id<T>());
    }
    
    // Should only allow conversion for objects that are actually of Python
    // type List or Tuple
    static void* convertible(PyObject* obj_ptr) {
        if(!(PyList_Check(obj_ptr) || PyTuple_Check(obj_ptr))) return 0; 
        
        //TODO: Add the call to the function to check that ever element
        //of the container is convertible
        PyObject *obj_itr = PyObject_GetIter(obj_ptr);
        if(!test_elements<T_v_t>::test(obj_itr)) return 0;

        return obj_ptr;
    }

    static void construct(
            PyObject* obj_ptr,
            boost::python::converter::rvalue_from_python_stage1_data* data)
    {
        void* storage = (
                (boost::python::converter::rvalue_from_python_storage<T>*)
                    data)->storage.bytes;
        new (storage) T();
        data->convertible = storage;
        T& result = *((T*)storage);

        PyObject *iterator = PyObject_GetIter(obj_ptr);
        PyObject *item;
        
        while(item = PyIter_Next(iterator)) {
            // extract the data we need
            T_v_t t = boost::python::extract<T_v_t>(item);
            result.push_back(t);
            Py_DECREF(item);
        }

        Py_DECREF(iterator);
    }
};


/* This converts any datatype that has a const_iterator
 * to a python list. An example use for a vector of ints:
 *     container_to_list<vector<int> >();
 */
template<typename T>
struct container_to_list 
{
    static PyObject* convert(const T &v) {
        //PyObject *ret = PyTuple_New(v.size());
        boost::python::list l;
        typedef typename T::const_iterator const_iter;

        for(const_iter i = v.begin(); i != v.end(); i++) {
            //PyTuple_SetItem(ret, i, );
            l.append(boost::python::object(*i));
        }
        return boost::python::incref(l.ptr());
    }
};




#ifdef VECTOR23
typedef boost::variant< double, int, float, Vector3, Vector2, char, fftw_complex > bv1;
#else
typedef boost::variant< double, int, float, char, fftw_complex > bv1;
#endif

/* This converts a vector<bv1> to a Python list.
 */
struct variant_vector_to_list
{
    static PyObject* convert(const vector<bv1> &v) {
        boost::python::list l;
        vector<bv1>::const_iterator const_iter;
        boost::python::object py_obj;

        for(const_iter = v.begin(); const_iter != v.end(); const_iter++) {
            py_obj = boost::apply_visitor(get_py_object(), (*const_iter));
            l.append(py_obj);
        }

        return boost::python::incref(l.ptr());
    }
};




#endif //__BASIN_CONTAINER_CONVERSION_H__

